@@ -683,7 +683,7 @@ link_addr(struct dhcpcd_ctx *ctx, struct interface *ifp, struct nlmsghdr *nlm)
 			rta = RTA_NEXT(rta, len);
 		}
 		ipv4_handleifa(ctx, nlm->nlmsg_type, NULL, ifp->name,
-		    &addr, &net, &brd, ifa->ifa_flags);
+		    &addr, &net, &brd);
 		break;
 #endif
 #ifdef INET6
@@ -699,7 +699,7 @@ link_addr(struct dhcpcd_ctx *ctx, struct interface *ifp, struct nlmsghdr *nlm)
 			rta = RTA_NEXT(rta, len);
 		}
 		ipv6_handleifa(ctx, nlm->nlmsg_type, NULL, ifp->name,
-		    &addr6, ifa->ifa_prefixlen, ifa->ifa_flags);
+		    &addr6, ifa->ifa_prefixlen);
 		break;
 #endif
 	}
@@ -1334,7 +1334,7 @@ if_readraw(__unused struct interface *ifp, int fd,
 	bytes = recvmsg(fd, &msg, 0);
 	if (bytes == -1)
 		return -1;
-	*flags = 0;
+	*flags = RAW_EOF; /* We only ever read one packet. */
 	if (bytes) {
 #ifdef PACKET_AUXDATA
 		for (cmsg = CMSG_FIRSTHDR(&msg);
@@ -1368,10 +1368,12 @@ if_address(unsigned char cmd, const struct ipv4_addr *addr)
 	nlm.ifa.ifa_index = addr->iface->index;
 	nlm.ifa.ifa_family = AF_INET;
 	nlm.ifa.ifa_prefixlen = inet_ntocidr(addr->mask);
+#if 0
 	/* This creates the aliased interface */
 	add_attr_l(&nlm.hdr, sizeof(nlm), IFA_LABEL,
 	    addr->iface->alias,
 	    (unsigned short)(strlen(addr->iface->alias) + 1));
+#endif
 	add_attr_l(&nlm.hdr, sizeof(nlm), IFA_LOCAL,
 	    &addr->addr.s_addr, sizeof(addr->addr.s_addr));
 	if (cmd == RTM_NEWADDR)
@@ -1388,8 +1390,6 @@ int
 if_route(unsigned char cmd, const struct rt *rt)
 {
 	struct nlmr nlm;
-	struct in_addr src_addr;
-	int subnet;
 
 	memset(&nlm, 0, sizeof(nlm));
 	nlm.hdr.nlmsg_len = NLMSG_LENGTH(sizeof(struct rtmsg));
@@ -1412,10 +1412,9 @@ if_route(unsigned char cmd, const struct rt *rt)
 
 	if (cmd == RTM_DELETE) {
 		nlm.rt.rtm_scope = RT_SCOPE_NOWHERE;
-		subnet = -1;
 	} else {
 		/* Subnet routes are RTPROT_KERNEL otherwise RTPROT_BOOT */
-		if ((subnet = ipv4_srcaddr(rt, &src_addr)) == 1)
+		if (rt->gate.s_addr == ntohl(INADDR_ANY))
 			nlm.rt.rtm_protocol = RTPROT_KERNEL;
 		else
 			nlm.rt.rtm_protocol = RTPROT_BOOT;
@@ -1437,9 +1436,9 @@ if_route(unsigned char cmd, const struct rt *rt)
 		if (rt->gate.s_addr != htonl(INADDR_ANY))
 			add_attr_l(&nlm.hdr, sizeof(nlm), RTA_GATEWAY,
 			    &rt->gate.s_addr, sizeof(rt->gate.s_addr));
-		if (subnet != -1) {
+		if (rt->src.s_addr != ntohl(INADDR_ANY)) {
 			add_attr_l(&nlm.hdr, sizeof(nlm), RTA_PREFSRC,
-			    &src_addr.s_addr, sizeof(src_addr.s_addr));
+			    &rt->src.s_addr, sizeof(rt->src.s_addr));
 		}
 		if (rt->mtu) {
 			char metricsbuf[32];
@@ -1495,8 +1494,7 @@ if_initrt(struct dhcpcd_ctx *ctx)
 }
 
 int
-if_addrflags(__unused const struct in_addr *addr,
-    __unused const struct interface *ifp)
+if_addrflags(__unused const struct ipv4_addr *addr)
 {
 
 	/* Linux has no support for IPv4 address flags */
@@ -1540,9 +1538,11 @@ if_address6(unsigned char cmd, const struct ipv6_addr *ia)
 
 	/* Add as /128 if no IFA_F_NOPREFIXROUTE ? */
 	nlm.ifa.ifa_prefixlen = ia->prefix_len;
+#if 0
 	/* This creates the aliased interface */
 	add_attr_l(&nlm.hdr, sizeof(nlm), IFA_LABEL,
 	    ia->iface->alias, (unsigned short)(strlen(ia->iface->alias) + 1));
+#endif
 	add_attr_l(&nlm.hdr, sizeof(nlm), IFA_LOCAL,
 	    &ia->addr.s6_addr, sizeof(ia->addr.s6_addr));
 
@@ -1620,15 +1620,26 @@ if_route6(unsigned char cmd, const struct rt6 *rt)
 	if (rt->metric)
 		add_attr_32(&nlm.hdr, sizeof(nlm), RTA_PRIORITY, rt->metric);
 
-	if (cmd != RTM_DELETE && rt->mtu) {
-		char metricsbuf[32];
-		struct rtattr *metrics = (void *)metricsbuf;
+	if (cmd == RTM_ADD || cmd == RTM_CHANGE) {
+#if 0
+		/* XXX This fails to work, why? */
+		if (!IN6_IS_ADDR_UNSPECIFIED(&rt->src)) {
+			add_attr_l(&nlm.hdr, sizeof(nlm), RTA_PREFSRC,
+			    &rt->src.s6_addr, sizeof(rt->src.s6_addr));
+		}
+#endif
+		if (rt->mtu) {
+			char metricsbuf[32];
+			struct rtattr *metrics = (void *)metricsbuf;
 
-		metrics->rta_type = RTA_METRICS;
-		metrics->rta_len = RTA_LENGTH(0);
-		rta_add_attr_32(metrics, sizeof(metricsbuf), RTAX_MTU, rt->mtu);
-		add_attr_l(&nlm.hdr, sizeof(nlm), RTA_METRICS,
-		    RTA_DATA(metrics), (unsigned short)RTA_PAYLOAD(metrics));
+			metrics->rta_type = RTA_METRICS;
+			metrics->rta_len = RTA_LENGTH(0);
+			rta_add_attr_32(metrics, sizeof(metricsbuf),
+			    RTAX_MTU, rt->mtu);
+			add_attr_l(&nlm.hdr, sizeof(nlm), RTA_METRICS,
+			    RTA_DATA(metrics),
+			    (unsigned short)RTA_PAYLOAD(metrics));
+		}
 	}
 
 	return send_netlink(rt->iface->ctx, NULL,
@@ -1665,7 +1676,7 @@ if_initrt6(struct dhcpcd_ctx *ctx)
 }
 
 int
-if_addrflags6(const struct in6_addr *addr, const struct interface *ifp)
+if_addrflags6(const struct ipv6_addr *ia)
 {
 	FILE *fp;
 	char *p, ifaddress[33], address[33], name[IF_NAMESIZE + 1];
@@ -1677,8 +1688,8 @@ if_addrflags6(const struct in6_addr *addr, const struct interface *ifp)
 		return -1;
 
 	p = ifaddress;
-	for (i = 0; i < (int)sizeof(addr->s6_addr); i++) {
-		p += snprintf(p, 3, "%.2x", addr->s6_addr[i]);
+	for (i = 0; i < (int)sizeof(ia->addr.s6_addr); i++) {
+		p += snprintf(p, 3, "%.2x", ia->addr.s6_addr[i]);
 	}
 	*p = '\0';
 
@@ -1687,10 +1698,10 @@ if_addrflags6(const struct in6_addr *addr, const struct interface *ifp)
 	{
 		if (strlen(address) != 32) {
 			fclose(fp);
-			errno = ENOTSUP;
+			errno = EINVAL;
 			return -1;
 		}
-		if (strcmp(name, ifp->name) == 0 &&
+		if (strcmp(name, ia->iface->name) == 0 &&
 		    strcmp(ifaddress, address) == 0)
 		{
 			fclose(fp);
